const { spawn } = require('child_process');
const { exec } = require('../db/database');

function searchSploit(term) {
  return new Promise((resolve, reject) => {
    const child = spawn('searchsploit', ['-j', '--id', `${term}`])

    let fullData = null
    child.stdout.on('data', (data) => {
      fullData += data.toString()
    })

    child.stderr.on('error', (stderr) => console.error('Standard Error: ', stderr))

    child.on('error', (error) => console.error(' Error: ', data))

    child.stdout.on('close', async () => {

      fullData = fullData.replaceAll('null', '');
      fullData = JSON.parse(fullData)?.RESULTS_EXPLOIT;

      exploits = await getExploitsDetailsFromMongo(fullData);

      if (!exploits.length) {
        resolve({
          message: 'No exploits available',
          data: []
        })
        return;
      }

      resolve({
        message: 'Exploits available',
        data: exploits
      })

    })

    child.on('exit', async (code, signal) => {
      if (code) console.log(`Process exit with code ${code}`);
      if (signal) console.log(`Process killed with signal ${signal}`);
    })
  })
}

async function getExploitsDetailsFromMongo(exploits) {
  try {
    const conditions = exploits.map(exploit => exploit['EDB-ID'])
    const result = await exec(async (conn) => {
      return await conn
        .db(process.env.MONGODB_NAME)
        .collection("exploits")
        .aggregate([
          {
            $match: {
              exploit_id: {
                $in: conditions
              }
            }
          },
          { $project: { _id: 0 } }
        ]).toArray();
    });
    return result
  } catch (error) {
    return error
  }
}

module.exports = {
  searchSploit
}
